; ##################
; ### COMMON LISP LIBRARY
; ##################

;
; Math helpers
;
(def inc (lambda (x) (+ x 1)));
(def dec (lambda (x) (- x 1)));
(def double (lambda (a) (* a 2)));

(def plus +);
(def minus -);
(def mult *);
(def div /);

(def fact (lambda (n)
	(if (eq? n 1)
		1
		(* n (fact (- n 1)))
	)
));

; OLD EQUAL VERSION
;##############
;(def equal? (lambda (x y)
;	(if (eq? x y)
;		t
;		(if (pair? x)
;			(if (pair? y)
;				(if (equal? (car x) (car y))
;					(equal? (cdr x) (cdr y))
;				nil)
;			nil)
;		nil)
;	)
;));

(def equal? (lambda (x y)
	(cond
		((eq? x y) t)
		((atom? x) nil)
		((atom? y) nil)
		((equal? (car x) (car y))
			(equal? (cdr x) (cdr y))
		)
	)
));

;
; Closure helpers
;
(def twice (lambda (f x) (f (f x))));
(def compose (lambda (f g) (lambda (x) (g (f x)))));
(def map (lambda (f l) 
								(if l
									(cons (f (car l))	(map f (cdr l)))
									l)
								));

;
; List helpers
;
(def first car);
(def second (compose cdr car));
(def third (compose cdr (compose cdr car)));
(def rest cdr);

;
; MISC
;
(def atom? (lambda (x) (null? (pair? x))));

(def what (lambda (x)
	(cond
		((null? x) nil)
		((number? x) number)
		((pair? x) pair)
		((closure? x) closure)
		((procedure? x) procedure-or-specialform)
		((symbol? x) symbol)
		(t failed-to-identify)
	)
));

(def = eq?);
(def lambda? closure?);

;
; MACROS
;
(def and (macro (p) (list if (first p) (second p) nil)));
(def or (macro (p) (list if (first p) t (second p))));

(def and*
	(macro (p)
		(if p
			(list if (first p) (cons and* (rest p)) nil)
			t
		)
	)
);

(def defqq
	(macro (p)
		(list def
			(first p)
			(list quote
				(second p)
			)
		)
	)
);

(def if*
	(macro (p)
		(cond 
			((eq? 2 (length p)) (list if (first p)	(second p) nil))
			((eq? 3 (length p)) (cons if p))
			(t (error bad-args))
		)
	)
);

;#################################

(def cadar
	(lambda (p)
		(car (cdr (car p)))
	)
);

(def caadr
	(lambda (p)
		(car (car (cdr p)))
	)
);

(def caddr
	(lambda (p)
		(car (cdr (cdr p)))
	)
);

(def cadr
	(lambda (p)
		(car (cdr p))
	)
);

(def cddar
	(lambda (p)
		(cdr (cdr (car p)))
	)
);

(def caddar
	(lambda (p)
		(car (cdr (cdr (car p))))
	)
);
