; #################
; # RECORDS LISP LIBRARY #
; #################
; Author: Thomas Wllert
; EMail: thomas@woellert.net
;
; Munich University of Applied Sciences, FB07 Computer Science
; Semantik von Programmiersprachen - Prof.Dr.Schiedermeier - Master 1.Semester

;################################################
; Main method for defining a new record with the given name and field list
; USAGE: (define-record <name> (<field1> <field2> ... <fieldN>))
; EXAMPLE: (define-record test (a b c))
(def define-record 
	(lambda (s l)
		(if (null? (list? l))
			(error ERR:define-record:second-argument-must-be-a-list)
			(prog
				(def-rec-first s l)
				(def-rec-second s l)
				(def-rec-third s l)
			)
		)
	)
);
;################################################

;
; HELPER METHODS FOR "DEFINE-RECORD"
;

;################################################
; Helper Method for (define-record [...]
; Defines the first function of the record: (def make-<name> [...])
; Do NOT call this method directly, use (define-record [...]) instead
; USAGE: (def-rec-first <name> (<field1> <field2> ... <fieldN>))
; EXAMPLE: (def-rec-first test (a b c))
(def def-rec-first
	(lambda (s paramL)
		(eval
			(quasiquote
				(let ((listlength (length paramL)))
					(def ,(symbol make- s)
						(lambda (valueL)
							(if (eq? (length valueL) listlength)
								(if (list? valueL)
									(list->vector (append (list s) valueL))
									(prog
										(print ERR:record-make:2nd Argument:)
										(println valueL)
										(error ERR:record-make:second-argument-is-not-a-list)
									)								
								)
								(prog
									(print ERR:record-make:ParameterLength:)
									(println (length valueL))
									(print ERR:record-make:ParmeterList:)
									(println valueL)
									(error ERR:record-make:number-of-parameters-does-not-match)	
								)
							)
						)
					)
				)
			)
		)
	)
);
;################################################
;################################################
; Helper Method for (define-record [...]
; Defines the second function of the record: (def <name>? <obj>)
; Do NOT call this method directly, use (define-record [...]) instead
; USAGE: (def-rec-second <name> (<field1> <field2> ... <fieldN>))
; EXAMPLE: (def-rec-second test (a b c))
(def def-rec-second
	(lambda (s paramL)
		(eval 
			(quasiquote 
				(let ((listlength (length paramL)))
					(def ,(symbol s ?) 
						(lambda (obj)
							(if (vector? obj)
								(if (equal? (vector? obj 0) s)
									(if (eq? (length obj) (+ listlength 1)) t nil)
								nil)
							nil)
						)
					)
				)
			)
		)
	)
);
;################################################
;################################################
; Helper Method for (define-record [...]
; Defines the third function of the record: (def <name>-><fieldName> <record>)
; Do NOT call this method directly, use (define-record [...]) instead
; USAGE: (def-rec-third <name> (<field1> <field2> ... <fieldN>))
; EXAMPLE: (def-rec-third test (a b c))
(def def-rec-third
	(lambda (s paramL)
		(def-rec-third-recursion s paramL 1)
	)
);

; Helper Method for def-rec-third, calls itself recursively
; for all fields in the record to actually create a (<name>-><field> <record)
; function for all of them.
; Do NOT call this method directly, use (define-record [...]) instead
; USAGE: (def-rec-third-recursion <name> <fieldList> <n>)
; To actually start the recursion begin with n=1
(def def-rec-third-recursion
	(lambda (s paramL n)
		(if paramL
			(prog
				(def-rec-third-helper s (first paramL) n)
				(def-rec-third-recursion s (rest paramL) (+ n 1))
			)
			nil
		)
	)
);

; Helper Method for def-rec-third, actually creates the 
; "<s>-><vs> <record>" function.
; Do NOT call this method directly, use (define-record [...]) instead
; USAGE: (def-rec-third-helper <name> <valuesymbol> <index>)
(def def-rec-third-helper
	(lambda (s vs n)
		(eval
			(quasiquote
				(def ,(symbol s -> vs)
					(lambda (obj)
						(if (,(symbol s ?) obj)
							(vector? obj n)
							(error ERR:invalid-record)
						)
					)
				)
			)
		)
	)
);
;################################################
