; ####################
; # VARIANT-CASE LISP LIBRARY #
; ####################
; Author: Thomas Wllert
; EMail: thomas@woellert.net
;
; Munich University of Applied Sciences, FB07 Computer Science
; Semantik von Programmiersprachen - Prof.Dr.Schiedermeier - Master 1.Semester

;################################################
; Evaluates a variant-case
; USAGE: (variant-case <record> ((<recname1> <fieldlist1> <consequent1>) [...] (<recnameN> <fieldlistN> <consequentN>)))
; EXAMPLE: (variant-case foo ((test (a b c) (+ a b)) (blubb (b) b)));
; EXAMPLE: (variant-case foo ((tes (a b c) (+ a b)) (blubb (b) b) (else (error error-case))));
(def variant-case
	(lambda (record caseList)
		(if (null? (list? caseList))
			(error ERR:variant-case:second-argument-must-be-a-list)
			(if (null? (defined? record))
				(error ERR:variant-case:first-argument-is-not-defined)
				(if caseList
					(if (variant-case-checker record (car (car caseList)))
						(variant-case-solve record (car (car caseList)) (cadar caseList) (caddar caseList))
						(if (null? (eq? (car (car caseList)) else))
							(variant-case record (rest caseList))
							(eval (cdr (car caseList)))
						)
					)
					nil
				)
			)
		)
	)
);
;################################################

;
; HELPER METHODS FOR "VARIANT-CASE"
;

;################################################
; Helper Method for (variant-case [...]
; Checks if the given record is of the given name
; Do NOT call this method directly, use "variant-case" instead
; USAGE: (variant-case-checker <record> <name>)
; EXAMPLE: (variant-case-checker foo test)
(def variant-case-checker
	(lambda (record name)
		(let ((method (symbol name ?)))
			(prog
				(print method)
				(if (defined? (eval method))
					((eval method) record)
					nil
				)
			)
		)
	)
);
;################################################
;################################################
; Helper Method for (variant-case [...]
; Recursively builds the proper environment for the given case.
; After that it evaluates the included consequent.
; Do NOT call this method directly, use "variant-case" instead
; USAGE: (variant-case-solve <record> <name> (<field1> <field2> [...] <fieldN>) consequent)
; EXAMPLE:  (variant-case-solve foo test (a b c) (+ a b))
(def variant-case-solve
	(lambda (record name paramList consequent)
		(eval
			(if paramList
				(quasiquote
					(letrec ,(variant-case-env-recursion record name paramList ())
						(eval consequent)
					)
				)
				nil
			)
		)
	)
);

; Helper Method for (variant-case-solve [...]
; Recursively build a list for the environment: ((name1 value1) (name2 value2) ... (nameN valueN))
; of the variables included in the param list. The value is taken from the given record
; Start it with a resultList of ()
; Do NOT call this method directly, use "variant-case" instead
; USAGE: (variant-case-env-recursion <record> <name> (<field1> <field2> [...] <fieldN>) ())
; EXAMPLE:  (variant-case-env-recursion foo test (a b c) ())
(def variant-case-env-recursion
	(lambda (record name paramList resultList)
		(if paramList
			(variant-case-env-recursion record name (rest paramList) 
				(append resultList 
					(list
						(quasiquote (,(first paramList) (quote ,(variant-case-grabber record name (first paramList)))))
					)
				)
			)
			resultList
		)
	)
);

; Helper Method for (variant-case-env-recursion [...]
; Grabs a specific variable from a given record
; Do NOT call this method directly, use "variant-case" instead
; USAGE: (variant-case-grabber <record> <name> <varname>)
; EXAMPLE: (variant-case-grabber foo test a)
(def variant-case-grabber
	(lambda (record recname varname)
		(let ((method (symbol recname -> varname)))
			(if (defined? (eval method))
				((eval method) record)
				nil
			)
		)
	)
);
;################################################
